package rotater

import (
	"gopkg.in/natefinch/lumberjack.v2"
	"io"
	"sync"
)

type CloseFunc func()

type WriteSyncer interface {
	io.Writer
	Sync() error
}

type RotateSyncer interface {
	io.WriteCloser
	SetOnClose(CloseFunc)
	Rotate() error
	Sync() error
}

type rotateSyncer struct {
	setOnceOnclose *sync.Once
	onClose        CloseFunc
	lumberjack.Logger
}

func newRotater(filename string, maxSize, maxBackup, maxDay int) RotateSyncer {
	return &rotateSyncer{
		setOnceOnclose: &sync.Once{},
		Logger: lumberjack.Logger{
			Filename:   filename,
			MaxSize:    maxSize, // megabytes
			MaxBackups: maxBackup,
			MaxAge:     maxDay, //days
			LocalTime:  false,
			Compress:   false,
		},
	}
}
func (r *rotateSyncer) SetOnClose(closeFunc CloseFunc) {
	r.setOnceOnclose.Do(func() {
		r.onClose = closeFunc
	})
}

func (r *rotateSyncer) Rotate() error {
	return r.Logger.Rotate()
}
func (r *rotateSyncer) Close() error {
	defer func() {
		if r.onClose != nil {
			r.onClose()
		}
	}()
	return r.Logger.Close()
}

func (r *rotateSyncer) Sync() error {
	return nil
}

func (s *rotateSyncer) Write(bs []byte) (int, error) {
	return s.Logger.Write(bs)
}
